@@ -19,27 +19,23 @@
 // macros, see http://tinyurl.com/ycwdxco; 'defined(bitor)'
 // has to be checked in a separate condition, otherwise GCC complains 
 // about 'bitor' being an alternative token
-#if defined(_MSC_VER) && !defined(__cplusplus)
-#ifndef __GCCXML__
+#if defined(_MSC_EXTENSIONS)
 #if defined(bitor)
 #   pragma push_macro("bitor")
 #   undef bitor
 #   define bitor(x)
 #endif
 #endif
-#endif
 
 #define AUX778076_OP_NAME   bitor_
 #define AUX778076_OP_PREFIX bitor
 #define AUX778076_OP_TOKEN  |
 #include <boost/mpl/aux_/arithmetic_op.hpp>
 
-#if defined(_MSC_VER) && !defined(__clang__) && !defined(__cplusplus)
-#ifndef __GCCXML__
+#if defined(_MSC_EXTENSIONS)
 #if defined(bitor)
 #   pragma pop_macro("bitor")
 #endif
 #endif
-#endif
 
 #endif // BOOST_MPL_BITOR_HPP_INCLUDED
